@@ -84,7 +84,7 @@ SUBTYPE_LISTENER_NOT_REGISTERED = "LISTENER_NOT_REGISTERED"
 SUBTYPE_MISSING_API_KEY = "MISSING_API_KEY"
 SUBTYPE_MISSING_BASE = "MISSING_BASE"
 SUBTYPE_MISSING_KEY = "MISSING_KEY"
-SUBTYPE_MISSING_MESSAGE = "MISSING_MESSAGE"
+SUBTYPE_MISSING_TITLE = "MISSING_TITLE"
 SUBTYPE_MISSING_MODULES = "MISSING_MODULES"
 SUBTYPE_MISSING_PATH = "MISSING_PATH"
 SUBTYPE_MISSING_PATH_URL = "MISSING_PATH_URL"
