@@ -2,8 +2,9 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Callable
+from collections.abc import Awaitable, Callable
 import socket
+from typing import Any
 from uuid import uuid4
 
 import aiohttp
@@ -11,26 +12,13 @@ import aiohttp
 from systembridgeconnector.base import Base
 from systembridgeconnector.const import (
     EVENT_API_KEY,
-    EVENT_APP_ICON,
-    EVENT_APP_NAME,
-    EVENT_BASE,
     EVENT_DATA,
-    EVENT_DIRECTORIES,
     EVENT_EVENT,
-    EVENT_FILE,
     EVENT_ID,
-    EVENT_KEY,
     EVENT_MESSAGE,
     EVENT_MODULE,
-    EVENT_MODULES,
-    EVENT_PATH,
     EVENT_SUBTYPE,
-    EVENT_TEXT,
-    EVENT_TIMEOUT,
-    EVENT_TITLE,
     EVENT_TYPE,
-    EVENT_URL,
-    EVENT_VERSION,
     MODEL_MAP,
     MODEL_RESPONSE,
     SUBTYPE_BAD_API_KEY,
@@ -61,10 +49,20 @@ from systembridgeconnector.exceptions import (
     ConnectionClosedException,
     ConnectionErrorException,
 )
+from systembridgeconnector.models.get_data import GetData
+from systembridgeconnector.models.keyboard_key import KeyboardKey
+from systembridgeconnector.models.keyboard_text import KeyboardText
 from systembridgeconnector.models.media_directories import MediaDirectories
 from systembridgeconnector.models.media_files import File as MediaFile, MediaFiles
+from systembridgeconnector.models.media_get_file import MediaGetFile
+from systembridgeconnector.models.media_get_files import MediaGetFiles
+from systembridgeconnector.models.notification import Notification
+from systembridgeconnector.models.open_path import OpenPath
+from systembridgeconnector.models.open_url import OpenUrl
+from systembridgeconnector.models.register_data_listener import RegisterDataListener
 from systembridgeconnector.models.request import Request
 from systembridgeconnector.models.response import Response
+from systembridgeconnector.models.update import Update
 
 
 class WebSocketClient(Base):
@@ -81,7 +79,7 @@ class WebSocketClient(Base):
         self._api_host = api_host
         self._api_port = api_port
         self._api_key = api_key
-        self._response_futures: dict[str, asyncio.Future] = {}
+        self._responses: dict[str, tuple[asyncio.Future, str | None]] = {}
         self._session: aiohttp.ClientSession | None = None
         self._websocket: aiohttp.ClientWebSocketResponse | None = None
 
@@ -93,6 +91,8 @@ class WebSocketClient(Base):
     async def _send_message(
         self,
         request: Request,
+        wait_for_response: bool = True,
+        response_type: str | None = None,
     ) -> Response:
         """Send a message to the WebSocket"""
         if not self.connected or self._websocket is None:
@@ -101,13 +101,21 @@ class WebSocketClient(Base):
         request.api_key = self._api_key
         request.id = uuid4().hex
         future: asyncio.Future[Response] = asyncio.get_running_loop().create_future()
-        self._response_futures[request.id] = future
+        self._responses[request.id] = future, response_type
         await self._websocket.send_str(request.json())
         self._logger.debug("Sent message: %s", request.json(exclude={EVENT_API_KEY}))
-        try:
-            return await future
-        finally:
-            self._response_futures.pop(request.id)
+        if wait_for_response:
+            try:
+                return await future
+            finally:
+                self._responses.pop(request.id)
+        return Response(
+            **{
+                EVENT_ID: request.id,
+                EVENT_TYPE: "N/A",
+                EVENT_MESSAGE: "Message sent",
+            }
+        )
 
     async def close(self) -> None:
         """Close connection"""
@@ -145,7 +153,7 @@ class WebSocketClient(Base):
 
     async def application_update(
         self,
-        version: str | None = None,
+        model: Update,
     ) -> Response:
         """Update application"""
         self._logger.info("Updating application")
@@ -153,9 +161,10 @@ class WebSocketClient(Base):
             Request(
                 **{
                     EVENT_EVENT: TYPE_APPLICATION_UPDATE,
-                    EVENT_VERSION: version,
+                    **model.dict(),
                 }
-            )
+            ),
+            wait_for_response=False,
         )
 
     async def exit_backend(self) -> Response:
@@ -166,22 +175,24 @@ class WebSocketClient(Base):
                 **{
                     EVENT_EVENT: TYPE_EXIT_APPLICATION,
                 }
-            )
+            ),
+            wait_for_response=False,
         )
 
     async def get_data(
         self,
-        modules: list[str],
+        model: GetData,
     ) -> Response:
         """Get data from server"""
-        self._logger.info("Getting data from server: %s", modules)
+        self._logger.info("Getting data from server: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_GET_DATA,
-                    EVENT_MODULES: modules,
+                    **model.dict(),
                 }
-            )
+            ),
+            response_type=TYPE_DATA_UPDATE,
         )
 
     async def get_directories(self) -> MediaDirectories:
@@ -194,138 +205,126 @@ class WebSocketClient(Base):
                 }
             )
         )
-        return MediaDirectories(**response.dict()[EVENT_DIRECTORIES])
+        return MediaDirectories(**response.dict())
 
     async def get_files(
         self,
-        base: str,
-        path: str | None = None,
+        model: MediaGetFiles,
     ) -> MediaFiles:
         """Get files"""
-        self._logger.info("Getting files: %s - %s", base, path)
+        self._logger.info("Getting files: %s", model.json())
         response = await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_GET_FILES,
-                    EVENT_BASE: base,
-                    EVENT_PATH: path,
+                    **model.dict(),
                 }
             )
         )
-        return MediaFiles(**response.dict()[EVENT_DIRECTORIES])
+        return MediaFiles(**response.dict())
 
     async def get_file(
         self,
-        base: str,
-        path: str,
+        model: MediaGetFile,
     ) -> MediaFile:
         """Get files"""
-        self._logger.info("Getting file: %s - %s", base, path)
+        self._logger.info("Getting file: %s", model.json())
         response = await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_GET_FILE,
-                    EVENT_BASE: base,
-                    EVENT_PATH: path,
+                    **model.dict(),
                 }
             )
         )
-        return MediaFile(**response.dict()[EVENT_FILE])
+        return MediaFile(**response.dict())
 
     async def register_data_listener(
         self,
-        modules: list[str],
+        model: RegisterDataListener,
     ) -> Response:
         """Register data listener"""
-        self._logger.info("Registering data listener: %s", modules)
+        self._logger.info("Registering data listener: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_REGISTER_DATA_LISTENER,
-                    EVENT_MODULES: modules,
+                    **model.dict(),
                 }
             )
         )
 
     async def keyboard_keypress(
         self,
-        key: str,
+        model: KeyboardKey,
     ) -> Response:
         """Keyboard keypress"""
-        self._logger.info("Press key: %s", key)
+        self._logger.info("Press key: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_KEYBOARD_KEYPRESS,
-                    EVENT_KEY: key,
+                    **model.dict(),
                 }
             )
         )
 
     async def keyboard_text(
         self,
-        text: str,
+        model: KeyboardText,
     ) -> Response:
         """Keyboard keypress"""
-        self._logger.info("Enter text: %s", text)
+        self._logger.info("Enter text: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_KEYBOARD_TEXT,
-                    EVENT_TEXT: text,
+                    **model.dict(),
                 }
             )
         )
 
     async def send_notification(
         self,
-        message: str,
-        title: str | None = None,
-        app_name: str | None = None,
-        app_icon: str | None = None,
-        timeout: int = 5,
+        model: Notification,
     ) -> Response:
         """Send notification"""
-        self._logger.info("Send notification: %s", message)
+        self._logger.info("Send notification: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_NOTIFICATION,
-                    EVENT_MESSAGE: message,
-                    EVENT_TITLE: title,
-                    EVENT_APP_NAME: app_name,
-                    EVENT_APP_ICON: app_icon,
-                    EVENT_TIMEOUT: timeout,
+                    **model.dict(),
                 }
             )
         )
 
     async def open_path(
         self,
-        path: str,
+        model: OpenPath,
     ) -> Response:
         """Open path"""
-        self._logger.info("Opening path: %s", path)
+        self._logger.info("Opening path: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_OPEN,
-                    EVENT_PATH: path,
+                    **model.dict(),
                 }
             )
         )
 
     async def open_url(
         self,
-        url: str,
+        model: OpenUrl,
     ) -> Response:
         """Open url"""
-        self._logger.info("Opening URL: %s", url)
+        self._logger.info("Opening URL: %s", model.json())
         return await self._send_message(
             Request(
                 **{
                     EVENT_EVENT: TYPE_OPEN,
-                    EVENT_URL: url,
+                    **model.dict(),
                 }
             )
         )
@@ -398,7 +397,7 @@ class WebSocketClient(Base):
 
     async def listen(
         self,
-        callback: Callable,
+        callback: Callable[[str, Any], Awaitable[None]] | None = None,
         accept_other_types: bool = False,
     ) -> None:
         """Listen for messages and map to modules"""
@@ -408,22 +407,39 @@ class WebSocketClient(Base):
             self._logger.debug("New message: %s", message[EVENT_TYPE])
 
             if message.get(EVENT_ID) is not None:
-                future = self._response_futures.get(message[EVENT_ID])
-                if future is not None:
-                    response = Response(**message)
-                    self._logger.info(
-                        "Response: %s",
-                        response.json(
-                            include={
-                                EVENT_ID,
-                                EVENT_TYPE,
-                                EVENT_SUBTYPE,
-                                EVENT_MESSAGE,
-                            },
-                            exclude_unset=True,
-                        ),
-                    )
-                    future.set_result(response)
+                response_tuple = self._responses.get(message[EVENT_ID])
+                if response_tuple is not None:
+                    future, response_type = response_tuple
+                    if (
+                        response_type is not None
+                        and response_type != message[EVENT_TYPE]
+                    ):
+                        self._logger.info(
+                            "Response type '%s' does not match requested type '%s'.",
+                            message[EVENT_TYPE],
+                            response_type,
+                        )
+                    else:
+                        response = Response(**message)
+                        self._logger.info(
+                            "Response: %s",
+                            response.json(
+                                include={
+                                    EVENT_ID,
+                                    EVENT_TYPE,
+                                    EVENT_SUBTYPE,
+                                    EVENT_MESSAGE,
+                                },
+                                exclude_unset=True,
+                            ),
+                        )
+                        try:
+                            future.set_result(response)
+                        except asyncio.InvalidStateError:
+                            self._logger.warning(
+                                "Future already set for response ID: %s",
+                                message[EVENT_ID],
+                            )
 
             if message[EVENT_TYPE] == TYPE_ERROR:
                 if message[EVENT_SUBTYPE] == SUBTYPE_LISTENER_ALREADY_REGISTERED:
@@ -441,15 +457,16 @@ class WebSocketClient(Base):
                 if model is None:
                     self._logger.warning("Unknown model: %s", message[EVENT_MODULE])
                 else:
-                    await callback(
-                        message[EVENT_MODULE],
-                        model(**message[EVENT_DATA]),
-                    )
+                    if callback is not None:
+                        await callback(
+                            message[EVENT_MODULE],
+                            model(**message[EVENT_DATA]),
+                        )
             else:
                 self._logger.debug("Other message: %s", message[EVENT_TYPE])
                 if accept_other_types:
                     model = MODEL_MAP.get(EVENT_TYPE, MODEL_MAP[MODEL_RESPONSE])
-                    if model is not None:
+                    if model is not None and callback is not None:
                         await callback(
                             message[EVENT_TYPE],
                             model(**message),
@@ -459,7 +476,7 @@ class WebSocketClient(Base):
 
     async def listen_for_messages(
         self,
-        callback: Callable,
+        callback: Callable[[dict[Any, Any]], Awaitable[None]],
     ) -> None:
         """Listen for messages"""
         if not self.connected:
